@@ -3,20 +3,25 @@
 from multiprocessing.pool import ThreadPool
 from typing import TYPE_CHECKING
 
-from .aur import find_aur_packages
-from .core import PackageSource
+from .args import parse_args
+from .aur import find_aur_packages, find_aur_provided_deps
 from .exceptions import (
     DependencyVersionMismatchError,
     PackagesNotFoundInAURError,
     PackagesNotFoundInRepoError,
 )
 from .i18n import translate
+from .logging_extras import create_logger
 from .pacman import PackageDB
-from .pprint import print_error
+from .pikaprint import print_error
+from .pikatypes import PackageSource
 from .version import VersionMatcher
 
 if TYPE_CHECKING:
-    from .aur import AURPackageInfo
+    from .pikatypes import AURPackageInfo
+
+
+logger = create_logger("aur_deps")
 
 
 def check_deps_versions(
@@ -33,10 +38,18 @@ def check_deps_versions(
     return PackageDB.get_not_found_local_packages(deps_lines)
 
 
-def get_aur_pkg_deps_and_version_matchers(aur_pkg: "AURPackageInfo") -> dict[str, VersionMatcher]:
+def get_aur_pkg_deps_and_version_matchers(
+        aur_pkg: "AURPackageInfo",
+        *,
+        skip_check_depends: bool = False,
+        skip_runtime_deps: bool = False,
+) -> dict[str, VersionMatcher]:
     deps: dict[str, VersionMatcher] = {}
     for dep_line in (
-            (aur_pkg.depends or []) + (aur_pkg.makedepends or []) + (aur_pkg.checkdepends or [])
+            (aur_pkg.depends or [])
+            + (aur_pkg.makedepends or [])
+            + (aur_pkg.runtimedepends if (not skip_runtime_deps and aur_pkg.runtimedepends) else [])
+            + (aur_pkg.checkdepends if (not skip_check_depends and aur_pkg.checkdepends) else [])
     ):
         version_matcher = VersionMatcher(dep_line, is_pkg_deps=True)
         name = version_matcher.pkg_name
@@ -51,13 +64,16 @@ def find_dep_graph_to(
         from_pkg: "AURPackageInfo",
         to_pkgs: "list[AURPackageInfo]",
         all_pkgs: "list[AURPackageInfo]",
+        *,
+        skip_check_depends: bool = False,
 ) -> "list[AURPackageInfo]":
     result: list[AURPackageInfo] = []
     if len(to_pkgs) == 1:
         possible_end_pkg = to_pkgs[0]
         possible_end_pkgs_deps = (
-            possible_end_pkg.depends + possible_end_pkg.checkdepends +
-            possible_end_pkg.makedepends
+            possible_end_pkg.depends
+            + ([] if skip_check_depends else possible_end_pkg.checkdepends)
+            + possible_end_pkg.makedepends
         )
         for name in [from_pkg.name, *from_pkg.provides]:
             if name in possible_end_pkgs_deps:
@@ -65,7 +81,11 @@ def find_dep_graph_to(
                 break
         for pkg in all_pkgs:
             for name in [pkg.name, *pkg.provides]:
-                if name in possible_end_pkgs_deps:
+                if (
+                        name in possible_end_pkgs_deps
+                ) and (
+                        from_pkg != pkg
+                ):
                     result += find_dep_graph_to(
                         from_pkg=from_pkg,
                         to_pkgs=[pkg],
@@ -82,7 +102,7 @@ def find_dep_graph_to(
     return result
 
 
-def handle_not_found_aur_pkgs(  # pylint: disable=too-many-locals
+def handle_not_found_aur_pkgs(
         aur_pkg_name: str,
         aur_pkgs_info: "list[AURPackageInfo]",
         not_found_aur_deps: list[str],
@@ -134,12 +154,14 @@ def handle_not_found_aur_pkgs(  # pylint: disable=too-many-locals
                     )
 
                 problem_packages_names.append(aur_pkg.name)
-                for dependant_pkg in find_dep_graph_to(
+                problem_packages_names.extend(
+                    dependant_pkg.name
+                    for dependant_pkg in find_dep_graph_to(
                         from_pkg=aur_pkg,
                         to_pkgs=requested_aur_pkgs_info,
                         all_pkgs=aur_pkgs_info,
-                ):
-                    problem_packages_names.append(dependant_pkg.name)
+                    )
+                )
                 break
 
     raise PackagesNotFoundInAURError(
@@ -204,6 +226,7 @@ def find_missing_deps_for_aur_pkg(
         aur_pkgs_info=aur_pkgs_info,
     )
     if not not_found_in_requested_pkgs:
+        logger.debug("find_missing_deps_for_aur_pkg: NOT not_found_in_requested_pkgs")
         return []
 
     # check among local pkgs:
@@ -213,6 +236,7 @@ def find_missing_deps_for_aur_pkg(
         source=PackageSource.LOCAL,
     )
     if not not_found_local_pkgs:
+        logger.debug("find_missing_deps_for_aur_pkg: NOT not_found_local_pkgs")
         return []
 
     # repo pkgs:
@@ -222,13 +246,19 @@ def find_missing_deps_for_aur_pkg(
         source=PackageSource.REPO,
     )
     if not not_found_repo_pkgs:
+        logger.debug("find_missing_deps_for_aur_pkg: NOT not_found_repo_pkgs")
         return []
 
     # try finding those packages in AUR
     aur_deps_info, not_found_aur_deps = find_aur_packages(
         not_found_repo_pkgs,
     )
-    # @TODO: find packages Provided by AUR packages
+    provided_aur_deps_info, not_found_aur_deps = find_aur_provided_deps(
+        not_found_aur_deps,
+        version_matchers=version_matchers,
+    )
+    aur_deps_info += provided_aur_deps_info
+
     handle_not_found_aur_pkgs(
         aur_pkg_name=aur_pkg_name,
         aur_pkgs_info=aur_pkgs_info,
@@ -237,28 +267,50 @@ def find_missing_deps_for_aur_pkg(
     )
 
     # check versions of found AUR packages:
+    logger.debug("find_missing_deps_for_aur_pkg: version_matchers={}", version_matchers)
     for aur_dep_info in aur_deps_info:
         aur_dep_name = aur_dep_info.name
-        version_matcher = version_matchers[aur_dep_name]
-        if not version_matcher(aur_dep_info.version):
-            raise DependencyVersionMismatchError(
-                version_found=aur_dep_info.version,
-                dependency_line=version_matcher.line,
-                who_depends=aur_pkg_name,
-                depends_on=aur_dep_name,
-                location=PackageSource.AUR,
-            )
+        version_matcher = version_matchers.get(
+            aur_dep_name,
+        )
+        pkg_version_matchers: list[VersionMatcher] = []
+        if version_matcher:
+            pkg_version_matchers = [version_matcher]
+        else:
+            for provide in aur_dep_info.provides:
+                version_matcher = version_matchers.get(VersionMatcher(provide).pkg_name)
+                if version_matcher is not None:
+                    pkg_version_matchers.append(version_matcher)
+        logger.debug(
+            "find_missing_deps_for_aur_pkg: {} pkg version_matchers={}",
+            aur_dep_name, pkg_version_matchers,
+        )
+        for version_matcher in pkg_version_matchers:
+            if not version_matcher(aur_dep_info.version):
+                raise DependencyVersionMismatchError(
+                    version_found=aur_dep_info.version,
+                    dependency_line=version_matcher.line,
+                    who_depends=aur_pkg_name,
+                    depends_on=aur_dep_name,
+                    location=PackageSource.AUR,
+                )
+            # not_found_repo_pkgs.remove(version_matcher.pkg_name)
 
     return not_found_repo_pkgs
 
 
-def find_aur_deps(aur_pkgs_infos: "list[AURPackageInfo]") -> dict[str, list[str]]:
-    # pylint: disable=too-many-locals,too-many-branches
+def find_aur_deps(  # pylint: disable=too-many-branches
+        aur_pkgs_infos: "list[AURPackageInfo]",
+        skip_checkdeps_for_pkgnames: list[str] | None = None,
+        *,
+        skip_runtime_deps: bool = False,
+) -> dict[str, list[str]]:
     new_aur_deps: list[str] = []
     package_names = [
         aur_pkg.name
         for aur_pkg in aur_pkgs_infos
     ]
+    logger.debug("find_aur_deps: package_names={}", package_names)
     result_aur_deps: dict[str, list[str]] = {}
 
     initial_pkg_infos = initial_pkg_infos2_todo = aur_pkgs_infos[:]  # @TODO: var name
@@ -271,10 +323,19 @@ def find_aur_deps(aur_pkgs_infos: "list[AURPackageInfo]") -> dict[str, list[str]
             initial_pkg_infos = []
         else:
             aur_pkgs_info, not_found_aur_pkgs = find_aur_packages(iter_package_names)
+            provided_aur_deps_info, not_found_aur_pkgs = find_aur_provided_deps(
+                not_found_aur_pkgs,
+            )
+            aur_pkgs_info += provided_aur_deps_info
         if not_found_aur_pkgs:
+            logger.debug("not_found_aur_pkgs={}", not_found_aur_pkgs)
             raise PackagesNotFoundInAURError(packages=not_found_aur_pkgs)
         for aur_pkg in aur_pkgs_info:
-            aur_pkg_deps = get_aur_pkg_deps_and_version_matchers(aur_pkg)
+            aur_pkg_deps = get_aur_pkg_deps_and_version_matchers(
+                aur_pkg,
+                skip_check_depends=aur_pkg.name in (skip_checkdeps_for_pkgnames or []),
+                skip_runtime_deps=skip_runtime_deps,
+            )
             if aur_pkg_deps:
                 all_deps_for_aur_packages[aur_pkg.name] = aur_pkg_deps
 
@@ -295,7 +356,14 @@ def find_aur_deps(aur_pkgs_infos: "list[AURPackageInfo]") -> dict[str, list[str]
             for aur_pkg_name, request in all_requests.items():
                 try:
                     results = request.get()
-                except Exception:
+                except DependencyVersionMismatchError as exc:
+                    if exc.who_depends not in parse_args().ignore:
+                        raise
+                except Exception as exc:
+                    logger.debug(
+                        "exception during aur search: {}: {}",
+                        exc.__class__.__name__, exc,
+                    )
                     print_error(translate(
                         "Can't resolve dependencies for AUR package '{pkg}':",
                     ).format(pkg=aur_pkg_name))
@@ -311,6 +379,7 @@ def find_aur_deps(aur_pkgs_infos: "list[AURPackageInfo]") -> dict[str, list[str]
                 new_aur_deps.append(pkg_name)
                 iter_package_names.append(pkg_name)
 
+    logger.debug("find_aur_deps: result_aur_deps={}", result_aur_deps)
     return result_aur_deps
 
 
@@ -327,10 +396,14 @@ def get_aur_deps_list(aur_pkgs_infos: "list[AURPackageInfo]") -> "list[AURPackag
 def _find_repo_deps_of_aur_pkg(
         aur_pkg: "AURPackageInfo",
         all_aur_pkgs: "list[AURPackageInfo]",
+        *,
+        skip_check_depends: bool = False,
 ) -> list[VersionMatcher]:
     new_deps_vms: list[VersionMatcher] = []
 
-    version_matchers = get_aur_pkg_deps_and_version_matchers(aur_pkg)
+    version_matchers = get_aur_pkg_deps_and_version_matchers(
+        aur_pkg, skip_check_depends=skip_check_depends,
+    )
 
     not_found_in_requested_pkgs = check_requested_pkgs(
         aur_pkg_name=aur_pkg.name,
@@ -355,11 +428,23 @@ def _find_repo_deps_of_aur_pkg(
     return new_deps_vms
 
 
-def find_repo_deps_of_aur_pkgs(aur_pkgs: "list[AURPackageInfo]") -> list[VersionMatcher]:
+def find_repo_deps_of_aur_pkgs(
+        aur_pkgs: "list[AURPackageInfo]",
+        skip_checkdeps_for_pkgnames: list[str],
+) -> list[VersionMatcher]:
     new_dep_names: list[VersionMatcher] = []
     with ThreadPool() as pool:
         results = [
-            pool.apply_async(_find_repo_deps_of_aur_pkg, (aur_pkg, aur_pkgs))
+            pool.apply_async(
+                _find_repo_deps_of_aur_pkg,
+                (
+                    aur_pkg,
+                    aur_pkgs,
+                ),
+                {
+                    "skip_check_depends": aur_pkg.name in (skip_checkdeps_for_pkgnames or []),
+                },
+            )
             for aur_pkg in aur_pkgs
         ]
         pool.close()
